# Dieses Beispiel soll nur zeigen, wie einfach man mit NumPy und
# Matzrizen arbeiten kann (ab Python-Version 3.6). NumPy gehört
# nicht zu Standard-Python. Deshalb muss das Paket <numpy> separat
# u.U. erst in der Eingabeaufforderung (Konsolefenster) von
# Windows geladen werden durch:
#    python -m pip install numpy
#
# Als Beispiel dient die Helmerttransformation aus: 
# Benning, 2. Auflage, Statistik in Geodäsie, Geoinformation 
# und Bauwesen, Wichmann Verlag, 2010, S. 264ff
# in gleicher Syntax

import numpy as np

print("----- Datenquelle: Benning, 2. Auflage, Statistik in Geodäsie, ")
print("----- Geoinformation und Bauwesen, Wichmann Verlag, 2010, S. 264ff")
print("----- 4-Parameter-Helmerttransformation")
print("")
print("Dieses harmlose Beispiel soll nur verdeutlichen, wie einfach")
print("Matrizenoperatioen in Python mit <NumPy> zu handhaben sind.")
print("")

yT= np.array([413.6, 377.6, 487.6, 706.4, 757.3, 456.2, 706.6, 686.2])
print("\ny\u1D40 (Koordinaten der Passpunkte im Zielsystem)\n", yT)

y=yT.T
print("\ny (Koordinaten der Passpunkte im Zielsystem)\n", y)

X = np.array([[1.,0.,-186.1850, -170.1375],
     [0.,1., 170.1375, -186.1850],
     [1.,0., 145.3350, -109.8275],
     [0.,1., 109.8275,  145.3350],
     [1.,0., -93.4750,  170.0625],
     [0.,1.,-170.0625,  -93.4750],
     [1.,0., 134.3250,  109.9025],
     [0.,1.,-109.9025,  134.3250]])
print("\nX (Funktionalmatrix)\n", X)

N = np.transpose(X) @ X
print("\nN = X\u1D40X (Normalgleichungsmatrix)\n", N)

print("\nX\u1D40X * \u03B2 =  X\u1D40 * y (Normalgleichungen)")

Ninv = np.linalg.inv(N)
print("\nN\u207B\u00B9 (Inverse der Normalgleichungsmatrix)\n", Ninv)

n = np.transpose(X) @ y
print("\nn = X\u1D40 * y (rechte Seite der Normalgleichungen)\n", n)

beta= Ninv @ n
print("\n\u03B2 = N\u207B\u00B9 * n (Transformationsparameter)\n", beta)

v= X @ beta - y
print("\nv = -e = X\u03B2 - y (Verbesserugen, -Residuenvektor)\n", v)

ee = v @ v
print("\ne\u1D40e (Summe v\u1D40v)\n", ee)

ss = ee/4
print("\ne\u1D40e/4 = s\u00B2/4  (Varianz der Gewichtseinheit)\n",ss)

s = np.sqrt(ss)
print("\ns = sqrt(e\u1D40e/4)  (Standardabweichung der Gewichtseinheit)\n",s)

H = X @ Ninv @ np.transpose(X)
print("\nH = XN\u207B\u00B9X\u1D40 (Projektionsmatrix)\n",H)
Diag = np.diag(H)
print("\nDiagonale der Projektionsmatrix H\n",Diag)

redunz = 1 - Diag
print("\nRedundanzen r = 1 - Diagonale(H)\n",redunz)

sigma =  Ninv * ss
print("\nKovarianzmatrix der Parameterschätzwerte \u03A3\u03B2 = N\u207B\u00B9*s\u00B2/4 \n",sigma)

diagsigma = np.diag(sigma)
print("\nVarianzen der Mittelwerte und Parameterschätzwerte aus \u03A3\u03B2 = N\u207B\u00B9*s\u00B2/4 \n",diagsigma)

sqrt_diagsigma = np.sqrt(diagsigma)
print("\nStandardabweichungen der Mittelwerte und Parameterschätzwerte aus \u03A3\u03B2 \n",sqrt_diagsigma)

sigmay = H * ss
print("\nKovarianzmatrix der Schätzwerte Y\u1D62, X\u1D62 = H*s\u00B2/4 \n",sigmay)

diagsigmay = np.diag(sigmay)
print("\nVarianzen der Schätzwerte Y\u1D62, X\u1D62 \n",diagsigmay)

sqrt_diagsigmay = np.sqrt(diagsigmay)
print("\nStandardabweichungen der der Schätzwerte Y\u1D62, X\u1D62 \n",sqrt_diagsigmay)

std_deviations = np.sqrt(np.diag(sigmay))
print("\nVersion2: Standardabweichungen der der Schätzwerte Y\u1D62, X\u1D62 \n",std_deviations)

correlation_sigmay = sigmay / (std_deviations[:, np.newaxis] * std_deviations)
print("\nKorrelationsmatrix der Schätzwerte Y\u1D62, X\u1D62,  H*s\u00B2/4 \n",correlation_sigmay)

Neu = np.array([[1.,0.,-16.9050, -10.1675],
     [0.,1., 10.1675, -16.9050]])
print("\nNeu  (zu transformierende Koordinaten im Startsystem)\n", Neu)

ziel = Neu @ beta
print("\nNeu * \u03B2  (Neukoordinaten)\n", ziel)

print("\nDie Varianzen und Standardabweichungen(s. Benning S. 263) der ")    
print("Neupunktkoordinaten wurden nicht berechnet, weil deren Aussagen")
print("wegen der Restklaffenverteilung unklar sind.\n")      
print("\n",60*'-')

print("\n typ: ",N.dtype)
print("\n", type(N))

print("\n",60*'-')

tt = np.array([[0.933,-0.309, -0.916],[-0.309,0.13, 0.313],[-0.916,0.313, 0.906]])
print("\nEin anderes Beispiel (s. auch Benning S. 169):\n", tt)

stdtt_deviations = np.sqrt(np.diag(tt))
print("\nStandardabweichungen der Schätzwerte \n", stdtt_deviations)

correlation_tt= tt / (stdtt_deviations[:, np.newaxis] * stdtt_deviations)
print("\nKorrelationsmatrix der Schätzwerte \n", correlation_tt)








